//----------------------------------------------------------------------------
//      _____
//     *     *
//    *____   *____
//   * *===*   *==*
//  *___*===*___**  AVNET
//       *======*
//        *====*
//----------------------------------------------------------------------------
//
// This design is the property of Avnet.  Publication of this
// design is not authorized without written consent from Avnet.
//
// Please direct any questions to:  technical.support@avnet.com
//
// Disclaimer:
//    Avnet, Inc. makes no warranty for the use of this code or design.
//    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
//    any errors, which may appear in this code, nor does it make a commitment
//    to update the information contained herein. Avnet, Inc specifically
//    disclaims any implied warranties of fitness for a particular purpose.
//                     Copyright(c) 2013 Avnet, Inc.
//                             All rights reserved.
//
//----------------------------------------------------------------------------
//
// Create Date:         May 14, 2013
// Module Name:         retarget.c
// Project Name:        Avnet Retarget Example
//
// Tool versions:       ARM DS-5 r14
//
// Description:         This file contains re-implementations of functions
//                      whose C library implementations rely on semihosting.
//                      This implementation is based upon example retarget
//                      code provided by ARM.
//
// Dependencies:
//
// Revision:            May 14, 2013: 1.00 Initial version
//                      Aug 14, 2013: 1.01 Updated for customer example
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// System Includes.

#include <stdio.h>

//----------------------------------------------------------------------------
// Local Includes.

#include "zynq_ps_uart.h"

/*
** Importing __use_no_semihosting ensures that our image doesn't link
** with any C Library code that makes direct use of semihosting.
**
** Build with STANDALONE to include this symbol.
*/

#pragma import(__use_no_semihosting)

/*
** Retargeted I/O
** ==============
** The following C library functions make use of semihosting
** to read or write characters to the debugger console: fputc(),
** fgetc(), and _ttywrch().  They must be retargeted to write to
** the model's UART.  __backspace() must also be retargeted
** with this layer to enable scanf().  See the Compiler and
** Libraries Guide.
*/

/*
** These must be defined to avoid linking in stdio.o from the
** C Library
*/

struct __FILE { int handle;   /* Add whatever you need here */};
FILE __stdout;
FILE __stdin;

/*
** __backspace must return the last char read to the stream
** fgetc() needs to keep a record of whether __backspace was
** called directly before it
*/
int last_char_read;
int backspace_called;


int fgetc(FILE *f)
{
    int8u tempch;
    tempch = zynq_uart_get_char();
    last_char_read = (int)tempch;       /* backspace must return this value */
    return tempch;
}


int fputc(int ch, FILE *f)
{
	int8u tempch = ch;

    /* Check to see if the character written is a line feed character. */
    if (tempch == 10)
    {
        /* Line feed characters should be prefaced with a carriage return
    	 * character.
    	 */
    	zynq_uart_put_char(13);
    }

    /* Output the specified character to the UART.
	 */
    zynq_uart_put_char(tempch);
    return ch;
}


void _ttywrch(int ch)
{
    unsigned char tempch = ch;

    /* Check to see if the character written is a line feed character. */
    if (tempch == 10)
    {
    	zynq_uart_put_char(13);
    }

    /* Output the specified character to the UART.
	 */
	zynq_uart_put_char(tempch);
}

/*
** The effect of __backspace() should be to return the last character
** read from the stream, such that a subsequent fgetc() will
** return the same character again.
*/

int __backspace(FILE *f)
{
    backspace_called = TRUE;
    return 1;
}

/* END of Retargeted I/O */

/*
** Exception Signaling and Handling
** ================================
** The C library implementations of ferror() uses semihosting directly
** and must therefore be retargeted.  This is a minimal reimplementation.
** _sys_exit() is called after the user's main() function has exited.  The C library
** implementation uses semihosting to report to the debugger that the application has
** finished executing.
*/

int ferror(FILE *f)
{
    return EOF;
}

void _sys_exit(int return_code)
{
    while(1);
}
