//----------------------------------------------------------------------------
//      _____
//     *     *
//    *____   *____
//   * *===*   *==*
//  *___*===*___**  AVNET
//       *======*
//        *====*
//----------------------------------------------------------------------------
//
// This design is the property of Avnet.  Publication of this
// design is not authorized without written consent from Avnet.
//
// Please direct any questions to:  technical.support@avnet.com
//
// Disclaimer:
//    Avnet, Inc. makes no warranty for the use of this code or design.
//    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
//    any errors, which may appear in this code, nor does it make a commitment
//    to update the information contained herein. Avnet, Inc specifically
//    disclaims any implied warranties of fitness for a particular purpose.
//                     Copyright(c) 2013 Avnet, Inc.
//                             All rights reserved.
//
//----------------------------------------------------------------------------
//
// Create Date:         May 14, 2013
// Module Name:         zynq_io.h
// Project Name:        Avnet Retarget Example
//
// Tool versions:       ARM DS-5 r14
//
// Description:         Text-based example application console for
//                      ZedBoard Getting Started Design with DS-5
//
// Dependencies:
//
// Revision:            May 14, 2013: 1.00 Initial version
//                      Aug 14, 2013: 1.01 Updated for customer example
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Local Includes.

#include "zynq_ps_uart.h"

//----------------------------------------------------------------------------
// Function Definitions.

//----------------------------------------------------------------------------
// Performs a UART transmit operation for data byte.
//
//     Arguments:
//
//         base_address
//             Target UART base address for performing the transmit operation.
//
//         data
//             Data to be transmitted by the UART.
//
//     Return:
//         None.
//----------------------------------------------------------------------------
void zynq_uart_transmit_byte(int32u base_address, int8u data)
{
	/* Check to see if the TX FIFO is full, wait until byte can be put in the
	 * FIFO.
	 */
	while ((zynq_read32(base_address + PS_UART_CHANNEL_STATUS_OFFSET) & PS_UART_CHANNEL_STATUS_TXFULL) == PS_UART_CHANNEL_STATUS_TXFULL);

	/* Write the data byte into the TX FIFO.
	 */
    zynq_write32(base_address + PS_UART_RX_TX_FIFO_OFFSET, data);
}

//----------------------------------------------------------------------------
// Performs a polled UART receive operation for one data byte.
//
//     Arguments:
//
//         base_address
//             Target UART base address for performing the receive operation.
//
//     Return:
//         Data byte read from the receive FIFO.
//----------------------------------------------------------------------------
int8u zynq_uart_receive_byte(int32u base_address)
{
	/* Poll against the channel status register, waiting until there is data
	 * available in the RX FIFO.
	 */
	while ((zynq_read32(base_address + PS_UART_CHANNEL_STATUS_OFFSET) & PS_UART_CHANNEL_STATUS_RXEMPTY) == PS_UART_CHANNEL_STATUS_RXEMPTY);

	/* Return the next byte received.
	 */
	return (zynq_read32(base_address + PS_UART_RX_TX_FIFO_OFFSET));
}

//----------------------------------------------------------------------------
// Performs initialization of the Zynq PS UART device.
//
//     Arguments:
//         None.
//
//     Return:
//         None.
//----------------------------------------------------------------------------
void zynq_uart_initialize(void)
{
	/* Placeholder for the current baud divisor value evaluation. */
	int8u current_baud_divider;
	/* Final value for baud divider value. */
	int8u final_baud_divider = 0;
	/* Calculated baud error percentage. */
	int32u baud_error_percentage;
	/* Target baud rate used for calculating generator and divider values. */
	int32u baud_rate = PS_UART_BAUD_RATE;
	/* Difference between the current calculated baud rate and the target
	 * baud rate. */
	int32u current_baud_error;
	/* Current calculated baud rate based upon the current calculated value
	 * for the baud rate generator. */
	int32u current_baud_rate;
	/* Current calculated value for the baud rate generator. */
	int32u current_baud_rate_gen_value;
	/* Final value calculated for baud rate generator. */
	int32u final_baud_rate_gen_value = 0;
	/* Final difference between the current calculated baud rate and the
	 * target baud rate. */
	int32u final_baud_error = 0xFFFFFFFF;
	/* Input clock speed to PS UART peripheral used for STDOUT. */
	int32u uart_input_clock_speed;

    /* Determine the input clock setting based upon the specified STDOUT
     * setting above.
     */
    if (STDOUT_BASEADDRESS == ZYNQ_PS_UART_0_BASEADDR)
    {
    	uart_input_clock_speed = ZYNQ_PS_UART_0_CLOCK_HZ;
    }
    else if (STDOUT_BASEADDRESS == ZYNQ_PS_UART_1_BASEADDR)
	{
    	uart_input_clock_speed = ZYNQ_PS_UART_1_CLOCK_HZ;
	}
    else
    {
    	/* No supported UART found. */
    	return;
    }

	/* Determine the best possible baud rate generator and divider settings by
	 * iterating through all possible combinations from 4 to 254.
	 *
	 */
	for (current_baud_divider = 4; current_baud_divider < 255; current_baud_divider++) {

		/*
		 * Calculate the current baud rate generator setting based upon the
		 * following formula:
		 *
		 * uart_input_clock_speed / (baud_rate * (current_baud_divider + 1))
		 */
		current_baud_rate_gen_value = uart_input_clock_speed / (baud_rate * (current_baud_divider + 1));

		/*
		 * Calculate the current baud rate from the current BRGR value
		 */
		current_baud_rate = uart_input_clock_speed / (current_baud_rate_gen_value * (current_baud_divider + 1));

		/*
		 * Avoid unsigned integer underflow
		 */
		if (baud_rate > current_baud_rate)
		{
			current_baud_error = baud_rate - current_baud_rate;
		}
		else
		{
			current_baud_error = current_baud_rate - baud_rate;
		}

		/* Filter the calculated baud rate closest to requested baud rate.
		 */
		if (final_baud_error > current_baud_error)
		{

			final_baud_rate_gen_value = current_baud_rate_gen_value;
			final_baud_divider = current_baud_divider;
			final_baud_error = current_baud_error;
		}
	}

	/* Make sure the best error rate does not exceed the allowable limit.
	 */
	baud_error_percentage = (final_baud_error * 100) / baud_rate;
	if (PS_UART_MAX_BAUD_ERROR_RATE < baud_error_percentage)
	{
		/* An acceptable amount of error rate cannot be achieved with the
		 * provided PS UART input clock and the available divider values.
		 */
		return;
	}

	/* Using the final valued determined above, set the Baud Rate Clock
	 * Divisor Value (CD) of Baud_rate_gen_reg0.  Set the Baud Rate Divider
	 * Value (BDIV) of Baud_rate_divider_reg0.
	 */
	zynq_write32(STDOUT_BASEADDRESS + PS_UART_BAUD_GEN_OFFSET, final_baud_rate_gen_value);
	zynq_write32(STDOUT_BASEADDRESS + PS_UART_BAUD_DIV_OFFSET, final_baud_divider);

	/* Set the PS UART for 8 data bits, 1 stop bit, no parity bits and source
	 * clock set to (uart_ref_clk / 1).
	 */
	zynq_write32(STDOUT_BASEADDRESS + PS_UART_MODE_OFFSET, 0x20);

	/* Enable PS UART send and receive by setting the transmit and receive
	 * enable bits and performing a reset on the Tx and Rx data paths.
	 */
	zynq_write32((STDOUT_BASEADDRESS + PS_UART_CONTROL_OFFSET), 0x17);

	return;
}

//----------------------------------------------------------------------------
// Implements the putchar function via UART transmit operation for data bytes.
//
//     Arguments:
//
//         character
//             The character byte that will be transmitted through the UART.
//
//     Return:
//         None.
//----------------------------------------------------------------------------
int8u zynq_uart_get_char(void)
{
	/* Receive a character from the UART. */
	return zynq_uart_receive_byte(STDOUT_BASEADDRESS);
}

//----------------------------------------------------------------------------
// Implements the putchar function via UART transmit operation for data bytes.
//
//     Arguments:
//
//         character
//             The character byte that will be transmitted through the UART.
//
//     Return:
//         None.
//----------------------------------------------------------------------------
void zynq_uart_put_char(char character)
{
	/* Send the character to the UART. */
	zynq_uart_transmit_byte(STDOUT_BASEADDRESS, character);
}
